/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Media.java,v 523e997730e9 2010/06/26 16:21:59 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.mobile.media;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import javax.annotation.concurrent.Immutable;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import org.openide.util.Lookup;
import it.tidalwave.util.As;
import it.tidalwave.util.AsException;
import it.tidalwave.util.Id;
import it.tidalwave.util.Key;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.TypeSafeMap;
import it.tidalwave.util.TypeSafeHashMap;
import it.tidalwave.mobile.util.AsSupport;
import it.tidalwave.mobile.util.Downloadable;
import it.tidalwave.mobile.util.DownloadableImpl;
import it.tidalwave.util.Removable;
import it.tidalwave.util.logging.Logger;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
@Immutable
public class Media extends AsSupport implements As, Lookup.Provider
  {
    private static final String CLASS = Media.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    public static final Class<Media> Media = Media.class;

    public static final String DC = "http://purl.org/dc/elements/1.1/";
    public static final String XX = "urn:bluebill-mobile:";
    public static final String O = "http://www.tidalwave.it/rdf/observation/2009/02/24#"; // FIXME

    // FIXME: use Dunlin Core ids
    public static final Key<Entity> AUTHOR = new Key<Entity>(DC + "creator");
    public static final Key<Entity> SOURCE = new Key<Entity>(DC + "publisher");
    public static final Key<Entity> LOCATION = new Key<Entity>(O + "madeAt");
    public static final Key<String> CALL_TYPE = new Key<String>(DC + "title");
    public static final Key<Date> DATE = new Key<Date>(DC + "date");
    public static final Key<Id> ID = new Key<Id>(DC + "identifier");
    public static final Key<URI> RES_URL = new Key<URI>(XX + "url");
    public static final Key<String> RIGHTS = new Key<String>(DC + "rights");
    public static final Key<String> MIME_TYPE = new Key<String>(XX + "mime-type");
    public static final Key<Integer> DURATION = new Key<Integer>(XX + "duration");
    public static final Key<String> FORMAT = new Key<String>(DC + "format");
    public static final Key<String> TYPE = new Key<String>(DC + "type");

    @Nonnull
    private final TypeSafeMap map;

    @CheckForNull
    private Downloadable downloadable;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public Media()
      {
        this(Collections.<Key<?>, Object>emptyMap());
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private Media (final @Nonnull Map<Key<?>, Object> map)
      {
        this.map = new TypeSafeHashMap(map);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T> T get (final @Nonnull Key<T> key)
      throws NotFoundException
      {
        return map.get(key);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean contains (final @Nonnull Key<?> item)
      {
        return map.containsKey(item);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Set<Key<?>> getKeys()
      {
        return map.getKeys();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnegative
    public int getSize()
      {
        return map.getSize();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Iterator<Object> iterator()
      {
        return map.iterator();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T> Media with (final @Nonnull Key<T> key, final @Nonnull T value)
      {
        if (key.equals(FORMAT))
          {
            final String[] parts = ((String)value).split(";");
            final String mimeType = parts[0];
            Media media = with(MIME_TYPE, mimeType);
            
            final String durationAsString = parts[1].replaceAll("s$", "").trim();
            
            if (!durationAsString.equals(""))
              {
                media = media.with(DURATION, Integer.valueOf(durationAsString));
              }

            return media;
          }

        final Map<Key<?>, Object> asMap = map.asMap();
        asMap.put(key, value);

        asMap.remove(FORMAT);

        if (asMap.containsKey(MIME_TYPE))
          {
            asMap.remove(TYPE);

            if (((String)asMap.get(MIME_TYPE)).startsWith("image/"))
              {
                asMap.put(TYPE, "http://purl.org/dc/dcmitype/StillImage");
              }
            else if(((String) asMap.get(MIME_TYPE)).startsWith("audio/"))
              {
                asMap.put(TYPE, "http://purl.org/dc/dcmitype/Sound");
              }
          }

        if (asMap.containsKey(MIME_TYPE) || asMap.containsKey(DURATION))
          {
            final StringBuilder buffer = new StringBuilder();
            final Key<?>[] keys = new Key<?>[] { MIME_TYPE, DURATION };

            String separator = "";

            for (final Key<?> k : keys)
              {
                buffer.append(separator);

                if (k.equals(MIME_TYPE))
                  {
                    buffer.append(asMap.get(k));
                    separator = "; ";
                  }
                else if (asMap.containsKey(k))
                  {
                    buffer.append(asMap.get(k)).append("s");
                    separator = "; ";
                  }
              }

            if (buffer.length() > 0)
              {
                asMap.put(FORMAT, buffer.toString());
              }
          }

        return new Media(asMap);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    @Override
    public <T> T as (final @Nonnull Class<T> type, final @Nonnull NotFoundBehaviour<T> nfb)
      {
        if (type.equals(Downloadable.class) || type.equals(Removable.class))
          {
            return super.as(type, new NotFoundBehaviour<T>()
              {
                @Nonnull
                public synchronized T run (final Throwable t)
                  {
                    if (downloadable == null)
                      {
                        try
                          {
                            downloadable = new DownloadableImpl(new URL(Media.this.get(ID).stringValue()));
                          }
                        catch (MalformedURLException e)
                          {
                            return nfb.run(new AsException(type, e));
                          }
                        catch (NotFoundException e)
                          {
                            return nfb.run(new AsException(type, e));
                          }
                      }

                    return type.cast(downloadable);
                  }
              });
          }

        return super.as(type, nfb);
      }
  }
